/*-------------------------------------------------------------------------+
| Copyright 2022 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.variability.ui.configure;

import static org.eclipse.jface.dialogs.IDialogConstants.OK_ID;
import static org.eclipse.swt.SWT.BEGINNING;
import static org.eclipse.swt.SWT.CENTER;
import static org.eclipse.swt.SWT.COLOR_RED;
import static org.eclipse.swt.SWT.FULL_SELECTION;
import static org.eclipse.swt.SWT.NONE;
import static org.fortiss.tooling.ext.variability.ui.editor.VariabilityEditorUtils.CONFIGURATION_TREETABLE_WIDTH;
import static org.fortiss.tooling.ext.variability.ui.editor.VariabilityEditorUtils.setUpVariantConfigurationTreeViewer;
import static org.fortiss.variability.bind.BindUtils.checkVariantConfiguration;

import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.variability.model.features.configuration.VariantConfiguration;
import org.fortiss.variability.util.Pair;

/**
 * Dialog window for configuring variants.
 * 
 * @author bayha
 */
public class ConfigureVariantDialog extends Dialog {

	/** {@link VariantConfiguration} to be created with this dialog. */
	protected VariantConfiguration varConfig;

	/** Tree viewer to display the feature tree for configuring. */
	protected TreeViewer treeViewer;

	/** Label to display messages about the currently edited configuration. */
	Label msgLabel;

	/** {@link EContentAdapter} to react on changes to varConfig. */
	protected EContentAdapter contentAdapter = new EContentAdapter() {
		/** {@inheritDoc} */
		@Override
		public void notifyChanged(Notification notification) {
			super.notifyChanged(notification);

			if(treeViewer != null && treeViewer.getControl() != null &&
					!treeViewer.getControl().isDisposed()) {
				treeViewer.refresh();
			} else {
				if(varConfig != null) {
					varConfig.eAdapters().remove(this);
				}
			}

			updateMessage();
		}
	};

	/** Constructor. */
	public ConfigureVariantDialog(Shell parentShell, VariantConfiguration varConfig) {
		super(parentShell);

		this.varConfig = varConfig;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean isResizable() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected Control createDialogArea(Composite parent) {
		Control ctrl = super.createDialogArea(parent);

		parent.setLayout(new GridLayout(1, false));

		msgLabel = new Label(parent, NONE);
		msgLabel.setSize(CONFIGURATION_TREETABLE_WIDTH, 25);
		msgLabel.setLayoutData(new GridData(BEGINNING, CENTER, true, false));
		msgLabel.setForeground(Display.getCurrent().getSystemColor(COLOR_RED));
		updateMessage();

		treeViewer = new TreeViewer((Composite)ctrl, FULL_SELECTION);

		setUpVariantConfigurationTreeViewer(treeViewer, varConfig);

		treeViewer.setInput(varConfig);
		treeViewer.expandAll();

		varConfig.eAdapters().add(contentAdapter);

		return ctrl;
	}

	/** {@inheritDoc} */
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);

		newShell.setText("Configure Variant to be Generated");
	}

	/**
	 * Retrieves the {@link VariantConfiguration} that is edited by this dialog.
	 * 
	 * @return The {@link VariantConfiguration} that is edited by this dialog.
	 */
	public VariantConfiguration getVariantConfiguration() {
		return varConfig;
	}

	/** Updated the (error) message and enables/disables the OK button. */
	private void updateMessage() {
		if(varConfig != null) {
			final Pair<String, List<EObject>> problem = checkVariantConfiguration(varConfig);
			final Button okButton = this.getButton(OK_ID);

			if(problem == null) {
				msgLabel.setText("");

				// No problem found -> configuration can be started.
				if(okButton != null) {
					okButton.setEnabled(true);
				}
			} else {
				msgLabel.setText(problem.first);
				msgLabel.requestLayout();

				// Problem found -> configuration cannot be started.
				if(okButton != null) {
					okButton.setEnabled(false);
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected Control createContents(Composite parent) {
		final Control contents = super.createContents(parent);

		// Update message after Buttons were created.
		updateMessage();

		return contents;
	}
}
