/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.variability.ui.editor;

import static java.util.Arrays.asList;
import static org.fortiss.variability.model.VariabilityModelElementFactory.createFeatureConfigurationForFeature;
import static org.fortiss.variability.model.VariabilityStaticImpl.OPTIONAL_ALTERNATIVE_NOT_SELECTED_STRING;
import static org.fortiss.variability.util.VariabilityUtils.checkAndDeselectFeatureConfiguration;
import static org.fortiss.variability.util.VariabilityUtils.selectFeatureConfiguration;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.presentation.ModelElementLabelProvider;
import org.fortiss.variability.model.features.AbstractAlternativeFeature;
import org.fortiss.variability.model.features.AbstractFeature;
import org.fortiss.variability.model.features.configuration.AlternativeFeatureConfiguration;
import org.fortiss.variability.model.features.configuration.CompositionalFeatureConfiguration;
import org.fortiss.variability.model.features.configuration.IFeatureConfiguration;
import org.fortiss.variability.model.features.configuration.VariantConfiguration;

/**
 * Editing support for selecting alternative features in the feature column
 * 
 * @author bayha
 */
public class AlternativeConfigurationEditingSupport extends EditingSupport {

	/** The {@link VariantConfiguration} for which the Alternative shall be configured. */
	private final VariantConfiguration variantConfiguration;

	/** The {@link Tree} from the Editor. */
	TreeViewer viewer;

	/** The {@link Tree} from the Editor. */
	Tree tree;

	/**
	 * Constructor.
	 * 
	 * @param treeViewer
	 *            The {@link TreeViewer}, this
	 *            {@link EditingSupport} is intended for.
	 * @param variantConfiguration
	 *            The {@link VariantConfiguration} for which the
	 *            Alternative shall be configured.
	 */
	public AlternativeConfigurationEditingSupport(VariantConfiguration variantConfiguration,
			TreeViewer treeViewer) {
		super(treeViewer);
		this.variantConfiguration = variantConfiguration;
		this.viewer = treeViewer;
		this.tree = treeViewer.getTree();
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {

		if(element instanceof AlternativeFeatureConfiguration) {
			ComboBoxViewerCellEditor editor =
					new ComboBoxViewerCellEditor((Composite)getViewer().getControl());
			editor.setLabelProvider(new ModelElementLabelProvider());
			editor.setContentProvider(new ArrayContentProvider() {
				/** {@inheritDoc} */
				@Override
				public Object[] getElements(Object inputElement) {
					if(inputElement instanceof AlternativeFeatureConfiguration) {
						AbstractFeature feature = ((AlternativeFeatureConfiguration)inputElement)
								.getFeatureReference();
						// If the AlternativeFeatureConfiguration is correct, the referenced
						// feature needs to be an AlternativeFeature.
						assert feature instanceof AbstractAlternativeFeature;

						EList<AbstractFeature> alternatives =
								((AbstractAlternativeFeature)feature).getAlternatives();
						List<Object> r = new ArrayList<Object>(alternatives);

						if(feature.isOptional()) {
							r.add(OPTIONAL_ALTERNATIVE_NOT_SELECTED_STRING);
						}

						return r.toArray();
					}
					return new Object[0];
				}
			});
			editor.setInput(element);
			return editor;
		}

		return new CheckboxCellEditor(tree);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		if(element instanceof AlternativeFeatureConfiguration) {
			AlternativeFeatureConfiguration afc = (AlternativeFeatureConfiguration)element;
			return afc.valueToString();
		} else if(element instanceof IFeatureConfiguration) {
			return ((IFeatureConfiguration<?>)element).isSelected();
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(final Object element, final Object value) {
		// Case distinction for the type of the assigned value
		if(value instanceof String) {
			setValueString(element, (String)value);

			return;
		}
		if(value instanceof AbstractFeature) {
			setValueForAbstractFeature(element, (AbstractFeature)value);

			return;
		}
		if(value instanceof Boolean) {
			setValueForBoolean(element, (boolean)value);

			return;
		}
	}

	/** Sets the new value on the given Boolean element. */
	private void setValueForBoolean(final Object element, final boolean value) {
		// Boolean can only be there for CompositionalFeatureConfiguration
		assert element instanceof CompositionalFeatureConfiguration;

		CompositionalFeatureConfiguration fc = (CompositionalFeatureConfiguration)element;

		// For persistent configurations we have to use the command stack service.
		// For temporal we must not use it:
		if(isContainedInProject(fc)) {
			ICommandStackService.getInstance().runAsCommand(this.variantConfiguration,
					new Runnable() {
						@Override
						public void run() {
							if(value) {
								selectFeatureConfiguration(fc);
								expandNewElement(fc);
							} else {
								checkAndDeselectFeatureConfiguration(fc);
							}
						}
					});
		} else {
			if(value) {
				selectFeatureConfiguration(fc);
				expandNewElement(fc);
			} else {
				checkAndDeselectFeatureConfiguration(fc);
			}
		}
	}

	/** Sets the new value on the given AbstractFeature element. */
	private void setValueForAbstractFeature(final Object element, final AbstractFeature value) {

		// AbstractFeatures can only be there of AlternativeFeatureConfigurations
		assert element instanceof AlternativeFeatureConfiguration;

		AlternativeFeatureConfiguration afc = (AlternativeFeatureConfiguration)element;
		// For persistent configurations we have to use the command stack service.
		// For temporal we must not use it:
		if(isContainedInProject(variantConfiguration)) {
			ICommandStackService.getInstance().runAsCommand(this.variantConfiguration,
					new Runnable() {
						@Override
						public void run() {
							final IFeatureConfiguration<?> altConf =
									createFeatureConfigurationForFeature(value);
							altConf.setSelected(true);

							afc.setSelectedConfiguration(altConf);
							afc.setSelected(true);
							expandNewElement(afc);
						}
					});
		} else {
			final IFeatureConfiguration<?> altConf = createFeatureConfigurationForFeature(value);
			altConf.setSelected(true);
			afc.setSelectedConfiguration(altConf);

			afc.setSelected(true);
			expandNewElement(afc);
		}
	}

	/** Sets the new value on the given AbstractFeature element. */
	private void setValueString(final Object element, final String value) {
		// The only String, that might be selected is OPTIONAL_FEATURE_DESELECTED_LABEL
		assert (value).equals(OPTIONAL_ALTERNATIVE_NOT_SELECTED_STRING);
		// ... which shall only be there for AlternativeFeatureConfiguration
		assert element instanceof AlternativeFeatureConfiguration;

		AlternativeFeatureConfiguration afc = (AlternativeFeatureConfiguration)element;
		// For persistent configurations we have to use the command stack service.
		// For temporal we must not use it:
		if(isContainedInProject(variantConfiguration)) {
			ICommandStackService.getInstance().runAsCommand(this.variantConfiguration,
					new Runnable() {
						@Override
						public void run() {
							afc.setSelected(false);
							afc.setSelectedConfiguration(null);
						}
					});
		} else {
			afc.setSelected(false);
			afc.setSelectedConfiguration(null);
		}
	}

	/**
	 * Checks for the given {@link EObject} whether it is contained in any
	 * containment tree.
	 * 
	 * @param eObj
	 *            The {@link EObject} to be checked for containment in a
	 *            containment tree.
	 * 
	 * @return 'true' if eObj is contained in a containment tree, 'false'
	 *         otherwise.
	 */
	private boolean isContainedInProject(EObject eObj) {
		return IPersistencyService.getInstance().getTopLevelElementFor(eObj) != null;
	}

	/**
	 * Expands the given newConf Element in the TreeViewer while keeping the current
	 * expansion level.
	 */
	private void expandNewElement(IFeatureConfiguration<?> newConf) {
		List<Object> expanded = new ArrayList<Object>();

		expanded.addAll(asList(viewer.getExpandedElements()));
		expanded.add(newConf);

		viewer.setExpandedElements(expanded.toArray());
	}
}
